/***************************************************************************
 *   Copyright (C) 2006 by André Lindhjem <belgarat@sdf.lonestar.org>,     *
 *                         Kjetil Holien <kjetil.holien@gmail.com>,        *
 *                         Terje Risa <terje.risa@gmail.com> &             *
 *                         Øyvind Nerbråten <oyvind@nerbraten.com>         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************
 *
 * \file libdaisy.c
 * \author André Lindhjem, Kjetil Holien, Terje Risa & Øyvind Nerbråten
 * \date 21.03.2006
 * 
 * Libdaisy is a toolkit for parsing and playing Daisy Digital Talking Books (DTB).
 */

 
#ifndef LIBDAISY_H_
#define LIBDAISY_H_

#include <pthread.h>

#ifdef __cplusplus
extern "C" {
#endif



/* ************************************************	*
 * 	Global type definitions and datastructures		*
 * ************************************************	*/
 
/**
 * This will need to be initialized and passed along to the daisy functions.
 */
typedef void *daisyplayer_t;

/**
 * The different sort of status messages libdaisy might return.
 */
enum daisy_status 
{
	DAISY_ERROR_UNKNOWN							= 0x0000,	/* unknown error */

	/* parser related errors */
	

	/* audio related errors */
	DAISY_ERROR_AUDIO_NOT_INITIALIZED			= 0x0200,	/* audio isn't (properly) initialized */
	DAISY_ERROR_AUDIO_CREATE_MMAP				= 0x0201,	/* could not read file into memmory */
	DAISY_ERROR_AUDIO_FSTAT						= 0x0202,	/* could not get file stats */
	DAISY_ERROR_AUDIO_OPEN						= 0x0203,	/* could not open file */
	DAISY_ERROR_AUDIO_FREE_MMAP					= 0x0204,	/* could not free file from memory */
	DAISY_ERROR_AUDIO_INITIATE_DATA				= 0x0205,	/* could not initiate data struct */
	DAISY_ERROR_AUDIO_NOT_PLAYING				= 0x0206,	/* state is not playing */
	DAISY_ERROR_AUDIO_NOT_STOPPED				= 0x0207,	/* state is not stopped */
	DAISY_ERROR_AUDIO_DATA_IS_NULL				= 0x0208,	/* data struct seems to be NULL */
	DAISY_ERROR_AUDIO_MALLOC					= 0x0209,	/* failure allocating memmory */
	DAISY_ERROR_AUDIO_PAUSED_WHILE_NOT_PLAYING	= 0x0210,	/* paused when not playing */
	DAISY_ERROR_AUDIO_STOPPED_WHILE_NOT_PLAYING	= 0x0211,	/* stopped when not playing */
	
	/* playback related errors */
	DAISY_ERROR_PLAYBACK_NO_TEXT_IN_SEGMENT		= 0x0301,	/* no text in segment */
	DAISY_ERROR_PLAYBACK_NO_AUDIO_IN_SEGMENT	= 0x0302,	/* no audio in segment */
	DAISY_ERROR_PLAYBACK_NO_DTB_LOADED			= 0x0303,	/* no daisy dtb loaded */
	DAISY_ERROR_PLAYBACK_SEEK_FAILED			= 0x0304,	/* seek failed */
	
	/* other errors */
	DAISY_ERROR_MISC_INIT_MUTEX					= 0x0400,	/* could not initialize mutex */
	
	/* playback related status messages */
	DAISY_END_OF_BOOK							= 0x1100	/* we're at the end of the book */
};

/**
 * The different seek operations supported by the engine.
 * @see daisy_seek
 */
enum daisy_seek_option
{
	DAISY_SEEK_PREV_CHAPTER = 1,
	DAISY_SEEK_PREV_PASSAGE = 2,
	DAISY_SEEK_NEXT_CHAPTER = 3,
	DAISY_SEEK_NEXT_PASSAGE = 4,
	DAISY_SEEK_TO_BEGINNING = 5
};

/**
 * The different types of book information.
 * @see daisy_get_info
 */
enum daisy_bookinfo_option
{
	DAISY_BOOKINFO_TITLETEXT = 1,
	DAISY_BOOKINFO_TITLEIMAGE = 2,
	DAISY_BOOKINFO_TOTALTIME = 3
};

/**
 * The different types of chapter information.
 * @see daisy_get_chapter_info
 */
enum daisy_chapter_info
{
	DAISY_CHAPTER_TITLE = 1,
	DAISY_CHAPTER_WEIGHT = 2	
}; 

/**
 * Struct for holding a Daisy DTB playback position. 
 * Meant for storing chapter (smilops) and passage (nodepos) jump positions.
 * Works as a bookmark. The integer values is equal to the chapter number and passage numer in the
 * playback sequence (smilpos = 1 & nodepos = 1 is the position for passage 1 in chapter 1).
 * @see daisy_get_position
 * @see daisy_goto_position
 */
 typedef struct {
 	int smilpos;
 	int nodepos;
 } daisy_position;



/* ************************************************	*
 * 			Global function declarations  		  	*
 * ************************************************	*/

/**
 * Initializes the daisy library.
 * It should be called before any attempt to use the daisy functionality.
 * @param data - a void pointer to any object or datastructure you may need in the callback functions. 
 * 				This data pointer will be available in all callback functions. You can e.g. pass along a GUI object in
 * 				c++ so that you can output the text from the callbackfunctions in the GUI. Set this parameter to NULL
 * 				if you don't need it.
 * @param l_cb_daisy_audio_done - a pointer to the function which will be called when an audio segment is done playing.
 * @param l_cb_daisy_audio_next - a pointer to the function which will be called when an new audio segment starts playing, supplying the duration of the segment in ms.
 * @param l_cb_daisy_text - a pointer to the function which will be called when an new audio segment starts playing, supplying the text corresponding to the audio.
 * @param l_cb_daisy_id - a pointer to the function which will be called when an new audio segment starts playing, supplying the id of the text passage in the xml file.
 * @param l_cb_daisy_error - a pointer to the function which will be called when an engine error occures.
 * @param l_cb_daisy_progress - a pointer to the function which will be called during playback, supplying the progress in ms.
 * @returns ::daisyplayer_t - the daisy data struct which must be passed along with all the API functions.
 */
daisyplayer_t daisy_init (void *data,
						  void (*l_cb_daisy_audio_done)	(void *),			/* (data, progress_ms_t ), */
						  void (*l_cb_daisy_audio_next)	(void *, unsigned long int),/* (data, void ), */
						  void (*l_cb_daisy_text)		(void *, void *),			/* (data, passage ), */
						  void (*l_cb_daisy_id)			(void *, void *),			/* (data, id ) */
						  void (*l_cb_daisy_error)		(void *, enum daisy_status, const char *daisy_status_msg),
						  void (*l_cb_daisy_progress)	(void *, long int)			/* (data, progress_ms) */
);

/**
 * Terminates the daisy library and frees memory used by it.
 * It should be called when daisy are no longer needed.
 * @param daisy - the daisy data struct which must be passed along with all the API functions.
 */
void daisy_term (daisyplayer_t daisy);

/**
 * Loads a new daisy book.
 * @param daisy - the daisy data struct which must be passed along with all the API functions.
 * @param path - a full path to the daisy dtb (ncc.* | *.ncx) to open.
 * @returns 1 in case of success and -1 in case of error.
 */
int daisy_load (daisyplayer_t daisy, char *path);

/**
 * Starts playback if a book is loaded.
 * @param daisy - the daisy data struct which must be passed along with all the API functions.
 * @returns 1 in case of success and -1 in case of error.
 */
int daisy_play (daisyplayer_t daisy);

/**
 * Seek operations which can be performed when a book is loaded.
 * @param daisy - the daisy data struct which must be passed along with all the API functions.
 * @param seek_option - a #daisy_seek_option.
 * @returns 0 if end_of_book, 1 in case of success and -1 in case of error.
 * @see daisy_seek_option
 */
int daisy_seek (daisyplayer_t daisy, int seek_option);

/**
 * Retrieves the current playback position.
 * @param daisy - the daisy data struct which must be passed along with all the API functions.
 * @returns a pointer to a daisy_position struct containing the chapter and passage positions, or NULL in
 * 			case of error. The struct must be deallocated by the caller.
 * @see daisy_position
 * @see daisy_goto_position
 */
daisy_position *daisy_get_position (daisyplayer_t daisy);

/**
 * Seeks to a playback position (bookmark) and continue playback from there.
 * @param daisy - the daisy data struct which must be passed along with all the API functions.
 * @param position - a pointer to a daisy_position struct with the chapter and passage position.
 * @returns 1 in case of success and -1 in case of error.
 * @see daisy_position
 * @see daisy_get_position
 */
int daisy_goto_position (daisyplayer_t daisy, daisy_position *position);

/**
 * Stops playback.
 * @param daisy - the daisy data struct which must be passed along with all the API functions.
 * @returns 1 in case of success. Has no other return values at this point.
 */
int daisy_stop (daisyplayer_t daisy);

/**
 * Toggle pause. Pauses playback if state is playing and continues playing of state is paused.
 * @param daisy - the daisy data struct which must be passed along with all the API functions.
 * @returns 1 in case of success and -1 in case of error.
 */
int daisy_pause (daisyplayer_t daisy);

/**
 * Retrieves book meta information.
 * @param daisy - the daisy struct which must be passed along with all the API functions.
 * @param value - a #daisy_bookinfo_option value which states what information to retrieve.
 * @returns the string if found, otherwise NULL. The string must be deallocated by the caller.
 * @see daisy_bookinfo_option
 */
char *daisy_get_info (daisyplayer_t daisy, int value);

/**
 * Retrieves the number of chapters in in the loaded Daisy DTB.
 * @param daisy - the daisy struct which must be passed along with all the API functions.
 * @returns the number of chapters, or -1 in case of error.
 */
int daisy_get_chapter_count (daisyplayer_t daisy);

/**
 * Retrieves information about a given chapter.
 * @param daisy - the daisy struct which must be passed along with all the API functions.
 * @param num - the chapter number to retrive information from (use daisy_get_chapter_count to get the
 * number of chapters available).
 * @param option - a #daisy_chapter_info which states what information to retrieve.
 * @returns a pointer to a string containing the information, or NULL in case of error.
 * @see daisy_chapter_info
 * @see daisy_get_chapter_count
 */
char *daisy_get_chapter_info (daisyplayer_t daisy, int num, int option);

#ifdef __cplusplus
}
#endif


#endif /*LIBDAISY_H_*/

